import * as csInterface from 'cep-interface';
var fs = require('fs');

export const invertKeyValues = (obj, fn) =>{
    return Object.keys(obj).reduce((acc, key) => {
        const val = fn ? fn(obj[key]) : obj[key];
        acc[val] = acc[val] || [];
        acc[val].push(key);
        return acc;
    }, {});
}
export const getKeyByValue = (object, value) => Object.keys(object).find(key => object[key] === value);
export const isObject = (data) => typeof data === 'object';
export const slugify = (text) => text
      .toString()                     // Cast to string
      .toLowerCase()                  // Convert the string to lowercase letters
      .normalize('NFD')       // The normalize() method returns the Unicode Normalization Form of a given string.
      .trim()                         // Remove whitespace from both sides of a string
      .replace(/\s+/g, '-')           // Replace spaces with -
      .replace(/[^\w\-]+/g, '')       // Remove all non-word chars
      .replace(/\-\-+/g, '-');        // Replace multiple - with single -
      
export const titleCase = (str) =>{
    str = str.toLowerCase().split(' ');
    for (var i = 0; i < str.length; i++) {
      str[i] = str[i].charAt(0).toUpperCase() + str[i].slice(1); 
    }
    return str.join(' ');
}
export const minMaxInput = (min, max, input) => Math.min(Math.max(input, min), max);

export const hexStringToRGBAArray = (string) => {
    // Always return an array, either empty or filled.
    var rgb = [];
    var hex;
    // Get an array of rgb(a) values.
    if (string.substr(0, 1) === '#') {
        /* HEX STRING */
        // If the first character is # we're dealing with a hex string. Get
        // an array of each character. This is more explicit than dealing
        // with the indices of a String object due to all other instances
        // of hex in this function.
        hex = color.substr(1).split('');
        if (hex.length === 3) {
            // If this is a 3-char color, e.g. #f00, make it 6 characters
            // by duplicating each one.
            hex = [hex[0], hex[0], hex[1], hex[1], hex[2], hex[2]];
        }
        if (hex.length === 6) {
            // Only deal with 6-char hex colors when computing the rgb
            // array. Anything else at this point has been passed
            // incorrectly.
            var i = 0;
            var x = 0;
            var hexStr;
            // Convert each hex pair (represented by hexStr) into a decimal
            // value to go in rgb[]. i is the rgb[] index whilst x is 2i,
            // which translates Array(3) to Array(6).
            while (i < 3) {
                hexStr = hex[x] + hex[x + 1];
                rgb[i] = parseInt(hexStr, 16);
                i += 1;
                x = i * 2;
            }
        }
    } 
    // Add or remove the alpha value.
    rgb.push(1);
    // Ensure all values in rgb are decimal numbers, not strings.
    for (var i = 0; i < rgb.length; i++) {
        rgb[i] = parseInt(rgb[i], 10);
    }
    return rgb;
}

export const RGBAArrayToHSLAArray = (rgba)=>{
    let r = rgba[0] / 255,
    g = rgba[1] / 255,
    b = rgba[2] / 255,
    a = rgba[3],
    
// find greatest and smallest channel values
    cmin = Math.min(r,g,b),
    cmax = Math.max(r,g,b), 
    delta = cmax - cmin,
    h = 0,
    s = 0,
    l = 0;

    // calculate hue
    // no difference
    if (delta == 0)
        h = 0;
    // red is max
    else if (cmax == r)
        h = ((g - b) / delta) % 6;
    // green is max
    else if (cmax == g)
        h = (b - r) / delta + 2;
    // blue is max
    else
        h = (r - g) / delta + 4;

    h = Math.round(h * 60);

    // make negative hues positive behind 360°
    if (h < 0)
        h += 360;
    
    // calculate lightness
    l = (cmax + cmin) / 2;

    // calculate saturation
    s = delta == 0 ? 0 : delta / (1 - Math.abs(2 * l - 1));

    // multiply l and s by 100
    s = +(s * 100).toFixed(1);
    l = +(l * 100).toFixed(1);
    return [h,s,l,a];
}


/**
* Evaluates Extendscript and Returns Error if thrown
* Credit: Justin Taylor
* 
* @param {!String} script ExtendScript Code to be executed
* @return {!Promise(resolve)} Returns promise with funciton result or error message
*/
const evalES = (script)=>{
	var csi = new csInterface();
	return new Promise(function (resolve, reject) {
		csi.evalScript('try{' + script + '}catch(e){e}', resolve);
	});
};
/**
* Parse all Command IDs for After Effects from dictionary file
* Credit: Justin Taylor
* 
* @return Menu ID object
*/

export const smallCss = ()=>{
    try{
        var data = fs.readFileSync('/Users/botoboto/Library/Application Support/Adobe/CEP/extensions/layerStyles/public/spectrum-medium.css', 'utf-8');
        var regex = /(\d+)px\;/ig;
        var newData = data.replace(regex, function(match, $1){ return Math.round(parseInt($1)* 0.85).toString() + "px;"});
        fs.writeFileSync('/Users/botoboto/Library/Application Support/Adobe/CEP/extensions/layerStyles/public/spectrum-medium-new.css', newData, 'utf-8');   
    }catch(e){
        alert(e);
    }
}
export const cmdIDs = ()=>{
	return new Promise(function (resolve, reject) {
		var stream = require('stream');
		var path = require('path');
		var fs = require('fs');
		var readline = require('readline');
		evalES('Folder.appPackage.fsName').then(function (appPackage) {
			appPackage = decodeURI(appPackage);
			var datFile = path.join(appPackage, '/Dictionaries/es_ES/after_effects_es_ES.dat');
			//console.log(datFile);
			var instream = fs.createReadStream(datFile, 'utf16le');
			var outstream = new stream;
			var rl = readline.createInterface(instream, outstream);
			var menuIDs = {};
			rl.on('line', function (line) {
				if (line.match(/\$\$\$\/AE\/MenuID/)) {
					line = line.substring(line.lastIndexOf('/') + 1);
					var data = line.match(/[a-z|A-Z|0-9]*\_[0-9]*/);
					if (data) {
						var name = data[0].match(/[a-z|A-Z|0-9]*\_/)[0].slice(0, -1);
						var number = data[0].match(/\_[0-9]*/)[0].substr(1);
						menuIDs[number] = name;
					}
				}
			});
			rl.on('close', function () {
				resolve(menuIDs);
			});
		});
	});
};